@@ -3,12 +3,15 @@ package Moose::Meta::Method::Augmented;
 use strict;
 use warnings;
 
-our $VERSION   = '0.93_03';
-$VERSION = eval $VERSION;
+our $VERSION   = '0.01';
 our $AUTHORITY = 'cpan:STEVAN';
 
 use base 'Moose::Meta::Method';
 
+use Sub::Name;
+
+use Carp qw(confess);
+
 sub new {
     my ( $class, %args ) = @_;
 
@@ -22,11 +25,11 @@ sub new {
     my $super = $meta->find_next_method_by_name($name);
 
     (defined $super)
-        || $meta->throw_error("You cannot augment '$name' because it has no super method", data => $name);
+        || confess "You cannot augment '$name' because it has no super method";
 
     my $_super_package = $super->package_name;
-    # BUT!,... if this is an overridden method ....
-    if ($super->isa('Moose::Meta::Method::Overridden')) {
+    # BUT!,... if this is an overriden method ....
+    if ($super->isa('Moose::Meta::Method::Overriden')) {
         # we need to be sure that we actually
         # find the next method, which is not
         # an 'override' method, the reason is
@@ -47,11 +50,7 @@ sub new {
     };
 
     # FIXME store additional attrs
-    $class->wrap(
-        $body,
-        package_name => $meta->name,
-        name         => $name
-    );
+    $class->wrap($body);
 }
 
 1;
@@ -66,48 +65,27 @@ Moose::Meta::Method::Augmented - A Moose Method metaclass for augmented methods
 
 =head1 DESCRIPTION
 
-This class implements method augmentation logic for the L<Moose>
-C<augment> keyword.
+This class implements method augmenting logic for the L<Moose> C<augment> keyword.
 
-The augmentation subroutine reference will be invoked explicitly using
-the C<inner> keyword from the parent class's method definition.
+This involves setting up C<inner> for the superclass body, and dispatching to
+the superclass from the normal body.
 
-=head1 INHERITANCE
-
-C<Moose::Meta::Method::Augmented> is a subclass of L<Moose::Meta::Method>.
+The subclass definition (the augmentation itself) will be invoked explicitly
+using the C<inner> keyword from the parent class's method definition.
 
 =head1 METHODS
 
 =over 4
 
-=item B<< Moose::Meta::Method::Augmented->new(%options) >>
-
-This constructs a new object. It accepts the following options:
-
-=over 8
-
-=item * class
-
-The metaclass object for the class in which the augmentation is being
-declared. This option is required.
-
-=item * name
-
-The name of the method which we are augmenting. This method must exist
-in one of the class's superclasses. This option is required.
-
-=item * method
-
-The subroutine reference which implements the augmentation. This
-option is required.
-
-=back
+=item B<new>
 
 =back
 
 =head1 BUGS
 
-See L<Moose/BUGS> for details on reporting bugs.
+All complex software has bugs lurking in it, and this module is no 
+exception. If you find a bug please either email me, or add the bug
+to cpan-RT.
 
 =head1 AUTHOR
 
@@ -115,7 +93,7 @@ Yuval Kogman E<lt>nothingmuch@cpan.orgE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2006-2010 by Infinity Interactive, Inc.
+Copyright 2006-2008 by Infinity Interactive, Inc.
 
 L<http://www.iinteractive.com>
 
