@@ -33,7 +33,7 @@ sub new {
     # needed
     weaken($self->{'$!associated_metaclass'});    
 
-    $self->intialize_body;
+    $self->initialize_body;
 
     return $self;    
 }
@@ -47,7 +47,7 @@ sub associated_metaclass { (shift)->{'$!associated_metaclass'} }
 
 sub is_needed { defined $_[0]->{'&!body'} ? 1 : 0 }
 
-sub intialize_body {
+sub initialize_body {
     my $self = shift;
     # TODO:
     # the %options should also include a both 
@@ -108,7 +108,7 @@ not particularly useful.
 
 =item B<is_needed>
 
-=item B<intialize_body>
+=item B<initialize_body>
 
 =item B<associated_metaclass>
 
